Twai

Struct Twai 

pub struct Twai<'d, Dm: DriverMode> { /* private fields */ }
An active TWAI peripheral in Normal Mode.

In this mode, the TWAI controller can transmit and receive messages including error signals (such as error and overload frames).

Implementations

Source

impl Twai<'_, Async>

pub async fn transmit_async( &mut self, frame: &EspTwaiFrame, ) -> Result<(), EspTwaiError>

Transmits an EspTwaiFrame asynchronously over the TWAI bus.

The transmission is aborted if the future is dropped. The technical reference manual does not specifiy if aborting the transmission also stops it, in case it is activly transmitting. Therefor it could be the case that even though the future is dropped, the frame was sent anyways.

If the bus cannot carry the frame (e.g. no other node acknowledges it), the transmission is aborted once the peripheral enters the error-passive or bus-off state and the future resolves to EspTwaiError::TransmissionAborted or EspTwaiError::BusOff, respectively.

pub async fn receive_async(&mut self) -> Result<EspTwaiFrame, EspTwaiError>

Receives an EspTwaiFrame asynchronously over the TWAI bus.

Source

impl<'d, Dm> Twai<'d, Dm>
where Dm: DriverMode,

pub fn stop(self) -> TwaiConfiguration<'d, Dm>

Stop the peripheral, putting it into reset mode and enabling reconfiguration.

pub fn receive_error_count(&self) -> u8

Returns the value of the receive error counter.

pub fn transmit_error_count(&self) -> u8

Returns the value of the transmit error counter.

pub fn is_bus_off(&self) -> bool

Check if the controller is in a bus off state.

pub fn initiate_recovery(&mut self)

Available on crate feature unstable only.

Initiates recovery from the bus-off state.

The peripheral recovers once it has observed 128 occurrences of 11 consecutive recessive (idle) bits on the bus. Use Self::is_bus_off to poll whether recovery has completed.

Does nothing if the peripheral is not in the bus-off state.

Stability

This API is marked as unstable and is only available when the unstable crate feature is enabled. This comes with no stability guarantees, and could be changed or removed at any time.

pub fn num_available_messages(&self) -> u8

Get the number of messages that the peripheral has available in the receive FIFO.

Note that this may not be the number of valid messages in the receive FIFO due to fifo overflow/overrun.

pub fn clear_receive_fifo(&self)

Clear the receive FIFO, discarding any valid, partial, or invalid packets.

This is typically used to clear an overrun receive FIFO.

TODO: Not sure if this needs to be guarded against Bus Off or other error states.

pub fn transmit(&mut self, frame: &EspTwaiFrame) -> Result<(), EspTwaiError>

Sends the specified EspTwaiFrame over the TWAI bus.

pub fn receive(&mut self) -> Result<EspTwaiFrame, EspTwaiError>

Receives a TWAI frame from the TWAI bus.

pub fn split(self) -> (TwaiRx<'d, Dm>, TwaiTx<'d, Dm>)

Consumes this Twai instance and splits it into transmitting and receiving halves.